@@ -90,21 +90,34 @@ _gsasl_cram_md5_client_step (Gsasl_session_ctx * cctx,
   Gsasl_client_callback_authentication_id cb_authentication_id;
   Gsasl_client_callback_password cb_password;
   GCRY_MD_HD md5h;
-  char *hash;
+  unsigned char *hash;
   int hash_len = gcry_md_get_algo_dlen (GCRY_MD_MD5);
   size_t len;
   char *tmp;
   int i;
   int res;
 
-  switch(state->step)
+  switch (state->step)
     {
     case 0:
+      state->step++;
+      if (input_len == 0)
+	{
+	  *output_len = 0;
+	  return GSASL_NEEDS_MORE;
+	}
+      /* fall through */
+
+    case 1:
+      if (input_len == 0)
+	return GSASL_MECHANISM_PARSE_ERROR;
+
       ctx = gsasl_client_ctx_get (cctx);
       if (ctx == NULL)
 	return GSASL_CANNOT_GET_CTX;
 
-      cb_authentication_id = gsasl_client_callback_authentication_id_get (ctx);
+      cb_authentication_id =
+	gsasl_client_callback_authentication_id_get (ctx);
       if (cb_authentication_id == NULL)
 	return GSASL_NEED_CLIENT_AUTHENTICATION_ID_CALLBACK;
 
@@ -112,14 +125,15 @@ _gsasl_cram_md5_client_step (Gsasl_session_ctx * cctx,
       if (cb_password == NULL)
 	return GSASL_NEED_CLIENT_PASSWORD_CALLBACK;
 
-      md5h = gcry_md_open (GCRY_MD_MD5, GCRY_MD_FLAG_SECURE|GCRY_MD_FLAG_HMAC);
+      md5h =
+	gcry_md_open (GCRY_MD_MD5, GCRY_MD_FLAG_SECURE | GCRY_MD_FLAG_HMAC);
       if (md5h == NULL)
 	return GSASL_GCRYPT_ERROR;
 
       /* XXX? password stored in callee's output buffer */
       len = *output_len;
       res = cb_password (cctx, output, &len);
-      if (res != GSASL_OK)
+      if (res != GSASL_OK && res != GSASL_NEEDS_MORE)
 	return res;
       tmp = stringprep_utf8_nfkc_normalize (output, len);
       if (tmp == NULL)
@@ -129,7 +143,7 @@ _gsasl_cram_md5_client_step (Gsasl_session_ctx * cctx,
       if (res != GCRYERR_SUCCESS)
 	return GSASL_GCRYPT_ERROR;
 
-      gcry_md_write (md5h, input, input_len);
+      gcry_md_write (md5h, /*XXX*/ (unsigned char *) input, input_len);
 
       hash = gcry_md_read (md5h, GCRY_MD_MD5);
       if (hash == NULL)
@@ -137,7 +151,7 @@ _gsasl_cram_md5_client_step (Gsasl_session_ctx * cctx,
 
       len = *output_len;
       res = cb_authentication_id (cctx, output, &len);
-      if (res != GSASL_OK)
+      if (res != GSASL_OK && res != GSASL_NEEDS_MORE)
 	return res;
       tmp = stringprep_utf8_nfkc_normalize (output, len);
       if (tmp == NULL)
@@ -158,7 +172,7 @@ _gsasl_cram_md5_client_step (Gsasl_session_ctx * cctx,
 	  output[len + 2 * i + 0] = HEXCHAR (hash[i] >> 4);
 	}
       *output_len = len + 2 * hash_len;
-      
+
       gcry_md_close (md5h);
 
       state->step++;
@@ -227,7 +241,8 @@ _gsasl_cram_md5_server_start (Gsasl_session_ctx * sctx, void **mech_data)
 
   strcpy (challenge, CHALLENGE_FORMAT);
 
-  gcry_randomize (challenge + 1, NUMBER_OF_XS, GCRY_WEAK_RANDOM);
+  gcry_randomize ((unsigned char *) challenge + 1,
+		  NUMBER_OF_XS, GCRY_WEAK_RANDOM);
 
   for (i = 0; i < NUMBER_OF_XS / 2; i++)
     {
@@ -253,6 +268,7 @@ _gsasl_cram_md5_server_step (Gsasl_session_ctx * sctx,
   Gsasl_server_callback_retrieve cb_retrieve;
   int hash_len = gcry_md_get_algo_dlen (GCRY_MD_MD5);
   char *username = NULL;
+  char *key = NULL;
   Gsasl_ctx *ctx;
   int res;
 
@@ -296,8 +312,8 @@ _gsasl_cram_md5_server_step (Gsasl_session_ctx * sctx,
       response = (char *) malloc (hash_len * 2 + 1);
       if (response == NULL)
 	{
-	  free (username);
-	  return GSASL_MALLOC_ERROR;
+	  res = GSASL_MALLOC_ERROR;
+	  goto done;
 	}
 
       memcpy (response, input + input_len - hash_len * 2, hash_len * 2);
@@ -310,56 +326,54 @@ _gsasl_cram_md5_server_step (Gsasl_session_ctx * sctx,
   else if (cb_retrieve)
     {
       GCRY_MD_HD md5h;
-      char *hash;
-      size_t len;
+      unsigned char *hash;
       size_t keylen;
-      char *key;
       char *normkey;
       int i;
 
-      md5h = gcry_md_open (GCRY_MD_MD5, GCRY_MD_FLAG_SECURE|GCRY_MD_FLAG_HMAC);
+      md5h =
+	gcry_md_open (GCRY_MD_MD5, GCRY_MD_FLAG_SECURE | GCRY_MD_FLAG_HMAC);
       if (md5h == NULL)
 	{
-	  free (username);
-	  return GSASL_GCRYPT_ERROR;
+	  res = GSASL_GCRYPT_ERROR;
+	  goto done;
 	}
 
       res = cb_retrieve (sctx, username, NULL, NULL, NULL, &keylen);
-      if (res != GSASL_OK)
-	return res;
+      if (res != GSASL_OK && res != GSASL_NEEDS_MORE)
+	goto done;
       key = malloc (keylen);
       if (key == NULL)
-	return GSASL_MALLOC_ERROR;
-      res = cb_retrieve (sctx, username, NULL, NULL, key, &keylen);
-      if (res != GSASL_OK)
 	{
-	  free (username);
-	  free (key);
-	  return res;
+	  res = GSASL_MALLOC_ERROR;
+	  goto done;
 	}
+      res = cb_retrieve (sctx, username, NULL, NULL, key, &keylen);
+      if (res != GSASL_OK && res != GSASL_NEEDS_MORE)
+	goto done;
       normkey = stringprep_utf8_nfkc_normalize (key, keylen);
-      free (key);
       if (normkey == NULL)
 	{
-	  free (username);
-	  return GSASL_UNICODE_NORMALIZATION_ERROR;
+	  res = GSASL_UNICODE_NORMALIZATION_ERROR;
+	  goto done;
 	}
 
       res = gcry_md_setkey (md5h, normkey, strlen (normkey));
       free (normkey);
       if (res != GCRYERR_SUCCESS)
 	{
-	  free (username);
-	  return GSASL_GCRYPT_ERROR;
+	  res = GSASL_GCRYPT_ERROR;
+	  goto done;
 	}
 
-      gcry_md_write (md5h, challenge, strlen (challenge));
+      gcry_md_write (md5h, /*XXX*/ (unsigned char *)challenge,
+		     strlen (challenge));
 
       hash = gcry_md_read (md5h, GCRY_MD_MD5);
       if (hash == NULL)
 	{
-	  free (username);
-	  return GSASL_GCRYPT_ERROR;
+	  res = GSASL_GCRYPT_ERROR;
+	  goto done;
 	}
 
       res = GSASL_OK;
@@ -371,7 +385,11 @@ _gsasl_cram_md5_server_step (Gsasl_session_ctx * sctx,
     }
 
   free (username);
+  if (key)
+    free (key);
+  *output_len = 0;
 
+done:
   return res;
 }
 
