@@ -19,35 +19,7 @@
  *
  */
 
-#include <stdio.h>
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
-#endif
-#ifdef HAVE_NETDB_H
-#include <netdb.h>
-#endif
-#ifdef HAVE_STDLIB_H
-#include <stdlib.h>
-#endif
-#ifdef HAVE_LOCALE_H
-#include <locale.h>
-#endif
-#if HAVE_STRING_H
-# if !STDC_HEADERS && HAVE_MEMORY_H
-#  include <memory.h>
-# endif
-# include <string.h>
-#endif
-#if HAVE_STRINGS_H
-# include <strings.h>
-#endif
-#include <argp.h>
-#include <gsasl.h>
-
-#include <stringprep.h>
+#include "internal.h"
 
 #define MAX_LINE_LENGTH BUFSIZ
 
@@ -87,26 +59,26 @@ utf8cpy (char *dst, size_t * dstlen, char *src, size_t srclen)
   int nonasciiflag = 0;
   char *p;
 
-  if (srclen != strlen(src))
+  if (srclen != strlen (src))
     return !GSASL_OK;
 
-  p = stringprep_locale_to_utf8(src);
+  p = stringprep_locale_to_utf8 (src);
   if (p)
     {
-      int len = strlen(p);
+      int len = strlen (p);
 
-      if (len > *dstlen)
+      if (dst && *dstlen < len)
 	return GSASL_TOO_SMALL_BUFFER;
       *dstlen = len;
       if (dst)
-	strcpy(dst, p);
+	strcpy (dst, p);
     }
   else
     {
       int i;
 
       fprintf (stderr, " ** failed to convert data from %s to UTF-8\n",
-	       stringprep_locale_charset());
+	       stringprep_locale_charset ());
       fprintf (stderr, " ** check the system locale configuration\n");
       fprintf (stderr, " ** treating input as ASCII\n");
 
@@ -278,7 +250,7 @@ client_callback_passcode (Gsasl_session_ctx * ctx, char *out, size_t * outlen)
 Gsasl_qop
 client_callback_qop (Gsasl_session_ctx * ctx, Gsasl_qop serverqops)
 {
-  if (serverqops & qop == 0)
+  if (!(serverqops & qop))
     fprintf (stderr,
 	     "Warning: Server QOPs %d does not include client QOP %d.\n",
 	     serverqops, qop);
@@ -390,8 +362,8 @@ server_callback_external (Gsasl_session_ctx * ctx)
 
 int
 server_callback_validate (Gsasl_session_ctx * ctx,
-			  char *authorization_id,
-			  char *authentication_id, char *password)
+			  const char *authorization_id,
+			  const char *authentication_id, const char *password)
 {
   char *data;
 
@@ -420,12 +392,27 @@ server_callback_validate (Gsasl_session_ctx * ctx,
 
 int
 server_callback_retrieve (Gsasl_session_ctx * ctx,
-			  char *authentication_id,
-			  char *authorization_id,
-			  char *realm, char *key, size_t * keylen)
+			  const char *authentication_id,
+			  const char *authorization_id,
+			  const char *realm, char *key, size_t * keylen)
 {
   int rc;
 
+  if (authentication_id && strlen (authentication_id) > 0)
+    printf ("Authentication ID: %s\n", authentication_id);
+  else
+    printf ("No authentication ID\n");
+
+  if (authorization_id && strlen (authorization_id) > 0)
+    printf ("Authorization ID: %s\n", authorization_id);
+  else
+    printf ("No authorization ID\n");
+
+  if (realm && strlen (realm) > 0)
+    printf ("Realm: %s\n", realm);
+  else
+    printf ("No realm\n");
+
   if (password == NULL)
     password = strdup (readline ("Enter password: "));
 
@@ -472,7 +459,8 @@ server_callback_service (Gsasl_session_ctx * ctx,
 
 int
 server_callback_gssapi (Gsasl_session_ctx * ctx,
-			char *client_name, char *authentication_id)
+			const char *client_name,
+			const char *authentication_id)
 {
   char *data;
 
